=== modified file 'bzrlib/http_transport.py'
--- bzrlib/http_transport.py	2005-08-18 02:21:56 +0000
+++ bzrlib/http_transport.py	2005-09-14 20:49:08 +0000
@@ -6,63 +6,52 @@
 from bzrlib.transport import Transport, register_transport, \
     TransportNotPossible, NoSuchFile, NonRelativePath, \
     TransportError
-import os, errno, time, asyncore
+import os, errno
 from cStringIO import StringIO
 import urllib2
 
-from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
-from bzrlib.trace import mutter
-
-from effbot.org.http_client import do_request
-
-class HttpTransportError(TransportError):
-    pass
-
-class simple_wait_consumer(object):
-    """This is a consumer object for effbot.org downloading.
-
-    Basically, it takes a management object, which it expects to
-    fill itself (eventually) if it waits long enough.
-    So it loops to wait for the download to finish, until it
-    finally gets closed.
-    """
-    def __init__(self, url, extra_headers=None):
-        self.buffer = StringIO()
-        self.done = False
-        self.ok = True
-        self.status = None
-        self.url = url
-        do_request(self.url, self)
-
-    def get(self):
-        self.finish()
-
-        # Try and break cyclical loops
-        if self.ok and (self.status is None or self.status[1] == '200'):
-            self.buffer.seek(0)
-            return self.buffer
+from errors import BzrError, BzrCheckError
+from branch import Branch, BZR_BRANCH_FORMAT
+from trace import mutter
+
+# velocitynet.com.au transparently proxies connections and thereby
+# breaks keep-alive -- sucks!
+
+
+ENABLE_URLGRABBER = True
+
+
+if ENABLE_URLGRABBER:
+    import urlgrabber
+    import urlgrabber.keepalive
+    import urlgrabber.grabber
+    urlgrabber.keepalive.DEBUG = 0
+    def get_url(path, compressed=False):
+        try:
+            url = path
+            if compressed:
+                url += '.gz'
+            mutter("grab url %s" % url)
+            url_f = urlgrabber.urlopen(url, keepalive=1, close_connection=0)
+            if not compressed:
+                return url_f
+            else:
+                return gzip.GzipFile(fileobj=StringIO(url_f.read()))
+        except urllib2.URLError, e:
+            raise BzrError("remote fetch failed: %r: %s" % (url, e))
+        except urlgrabber.grabber.URLGrabError, e:
+            raise BzrError("remote fetch failed: %r: %s" % (url, e))
+else:
+    def get_url(url, compressed=False):
+        import urllib2
+        if compressed:
+            url += '.gz'
+        mutter("get_url %s" % url)
+        url_f = urllib2.urlopen(url)
+        if compressed:
+            return gzip.GzipFile(fileobj=StringIO(url_f.read()))
         else:
-            raise NoSuchFile('Download of %r failed: %r' 
-                    % (self.url, self.status))
-
-    def finish(self):
-        while not self.done:
-            asyncore.poll(0.1)
-
-    def feed(self, data):
-        self.buffer.write(data)
-
-    def close(self):
-        self.done = True
-
-    def http(self, ok, connection, *args, **kwargs):
-        mutter('simple-wait-consumer: %s, %s, %s, %s' 
-                % (self.url, ok, connection.status, connection.header))
-        self.ok = ok
-        if not ok:
-            self.done = True
-        self.status = connection.status
+            return url_f
 
 def _find_remote_root(url):
     """Return the prefix URL that corresponds to the branch root."""
@@ -103,6 +92,9 @@
         """Set the base path where files will be stored."""
         assert base.startswith('http://') or base.startswith('https://')
         super(HttpTransport, self).__init__(base)
+        # In the future we might actually connect to the remote host
+        # rather than using get_url
+        # self._connection = None
 
     def should_cache(self):
         """Return True if the data pulled across should be cached locally.
@@ -141,42 +133,33 @@
         TODO: HttpTransport.has() should use a HEAD request,
         not a full GET request.
         """
-        c = simple_wait_consumer(self.abspath(relpath))
-        c.finish()
-
-        # We might also look at c.status, but it doesn't seem to always be set
-        # c.status is of the form ["HTTP/1.0", "200", "OK\r\n"] we could
-        # check both the 200 and the OK.
-        if c.ok and (c.status is None or c.status[1] == '200'):
+        try:
+            f = get_url(self.abspath(relpath))
             return True
-        return False
+        except BzrError:
+            return False
+        except urllib2.URLError:
+            return False
+        except IOError, e:
+            if e.errno == errno.ENOENT:
+                return False
+            raise HttpTransportError(orig_error=e)
 
-    def get(self, relpath):
+    def get(self, relpath, decode=False):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
         """
-        c = simple_wait_consumer(self.abspath(relpath))
-        return c.get()
-
-    ### def get_multi(self, relpaths, pb=None):
-    ###     """Get a list of file-like objects, one for each entry in relpaths.
-
-    ###     :param relpaths: A list of relative paths.
-    ###     :param decode:  If True, assume the file is utf-8 encoded and
-    ###                     decode it into Unicode
-    ###     :param pb:  An optional ProgressBar for indicating percent done.
-    ###     :return: A list or generator of file-like objects
-    ###     """
-    ###     consumers = []
-    ###     for relpath in relpaths:
-    ###         consumers.append(simple_wait_consumer(self.abspath(relpath)))
-    ###     total = self._get_total(consumers)
-    ###     count = 0
-    ###     for c in consumers:
-    ###         self._update_pb(pb, 'get', count, total)
-    ###         yield c.get()
-    ###         count += 1
+        try:
+            return get_url(self.abspath(relpath))
+        except BzrError, e:
+            raise NoSuchFile(orig_error=e)
+        except urllib2.URLError, e:
+            raise NoSuchFile(orig_error=e)
+        except IOError, e:
+            raise NoSuchFile(orig_error=e)
+        except Exception,e:
+            raise HttpTransportError(orig_error=e)
 
     def put(self, relpath, f):
         """Copy the file-like or string object into the location.

=== modified file 'bzrlib/transport.py'
--- bzrlib/transport.py	2005-08-02 07:12:15 +0000
+++ bzrlib/transport.py	2005-09-14 20:49:08 +0000
@@ -322,6 +322,7 @@
 
         return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
 
+
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""
         raise NotImplementedError

=== modified file 'effbot/org/http_client.py'
--- effbot/org/http_client.py	2005-08-18 02:21:56 +0000
+++ effbot/org/http_client.py	2005-09-14 20:49:08 +0000
@@ -36,25 +36,32 @@
     def __init__(self, location):
         self.location = location
 
-class Request(object):
-    """This keeps track of all the information for a single request.
-    """
+##
+# Asynchronous HTTP/1.1 client.
+
+class async_http(asyncore.dispatcher_with_send):
+    # asynchronous http client
 
     user_agent = "http_client.py 1.2 (http://effbot.org/zone)"
     http_version = "1.1"
+
     proxies = urllib.getproxies()
 
     def __init__(self, uri, consumer, extra_headers=None):
-        self.consumer = consumer
+        asyncore.dispatcher_with_send.__init__(self)
+
         # turn the uri into a valid request
         scheme, host, path, params, query, fragment = urlparse.urlparse(uri)
 
-        self.scheme = scheme
+        # use origin host
         self.host = host
 
         # get proxy settings, if any
         proxy = self.proxies.get(scheme)
-        self.proxy = proxy
+        if proxy:
+            scheme, host, x, x, x, x = urlparse.urlparse(proxy)
+
+        assert scheme == "http", "only supports HTTP requests (%s)" % scheme
 
         if not path:
             path = "/"
@@ -67,11 +74,15 @@
 
         self.path = path
 
-        # It turns out Content-Length isn't sufficient
-        # to allow pipelining. Simply required.
-        # So we will be extra stingy, and require the
-        # response to also be HTTP/1.1 to enable pipelining
-        self.http_1_1 = False
+        # get port number
+        try:
+            host, port = host.split(":", 1)
+            port = int(port)
+        except (TypeError, ValueError):
+            port = 80 # default port
+
+        self.consumer = consumer
+
         self.status = None
         self.header = None
 
@@ -90,10 +101,14 @@
         self.timestamp = time.time()
 
         self.extra_headers = extra_headers
-        self.requested = False
-
-    def http_request(self, conn):
-        """Place the actual http request on the server"""
+
+        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
+        try:
+            self.connect((host, port))
+        except socket.error:
+            self.consumer.http(0, self, sys.exc_info())
+
+    def handle_connect(self):
         # connection succeeded
 
         request = [
@@ -101,7 +116,7 @@
             "Host: %s" % self.host,
             ]
 
-        if False and GzipConsumer:
+        if GzipConsumer:
             request.append("Accept-Encoding: gzip")
 
         if self.extra_headers:
@@ -116,18 +131,21 @@
 
         request = string.join(request, "\r\n") + "\r\n\r\n"
 
-        conn.send(request)
+        self.send(request)
 
         self.bytes_out = self.bytes_out + len(request)
-        self.requested = True
-
-    def add_data(self, data):
-        """Some data has been downloaded, let the consumer know.
-        :return: True- download is completed, there may be extra data in self.data which
-                       should be transfered to the next item
-                 False- download failed
-                 None- continue downloading
-        """
+
+    def handle_expt(self):
+        # connection failed (windows); notify consumer
+
+        if sys.platform == "win32":
+            self.close()
+            self.consumer.http(0, self)
+
+    def handle_read(self):
+        # handle incoming data
+
+        data = self.recv(2048)
 
         self.data = self.data + data
         self.bytes_in = self.bytes_in + len(data)
@@ -147,11 +165,6 @@
                 self.status = fp.readline().split(" ", 2)
                 self.header = mimetools.Message(fp)
 
-                if self.status[0] == 'HTTP/1.1':
-                    self.http_1_1 = True
-                else:
-                    self.http_1_1 = False
-
                 # get http headers
                 self.content_type = self.header.get("content-type")
                 try:
@@ -160,13 +173,12 @@
                         )
                 except (ValueError, TypeError):
                     self.content_length = None
-                self.original_content_length = self.content_length
                 self.transfer_encoding = self.header.get("transfer-encoding")
                 self.content_encoding = self.header.get("content-encoding")
 
-                # if self.content_encoding == "gzip":
-                #     # FIXME: report error if GzipConsumer is not available
-                #     self.consumer = GzipConsumer(self.consumer)
+                if self.content_encoding == "gzip":
+                    # FIXME: report error if GzipConsumer is not available
+                    self.consumer = GzipConsumer(self.consumer)
 
                 try:
                     self.consumer.http(1, self)
@@ -176,9 +188,11 @@
                         do_request(
                             v.location, self.consumer, self.extra_headers
                             )
-                    return True
+                    self.close()
+                    return
                 except CloseConnection:
-                    return False
+                    self.close()
+                    return
 
             if self.transfer_encoding == "chunked" and self.chunk_size is None:
 
@@ -196,8 +210,7 @@
                     if self.chunk_size <= 0:
                         raise ValueError
                 except ValueError:
-                    self.consumer.close()
-                    return False
+                    return self.handle_close()
 
             if not self.data:
                 return
@@ -207,12 +220,6 @@
 
             chunk_size = self.chunk_size or len(data)
 
-            # Make sure to only feed the consumer whatever is left for
-            # this file.
-            if self.content_length:
-                if chunk_size > self.content_length:
-                    chunk_size = self.content_length
-
             if chunk_size < len(data):
                 self.data = data[chunk_size:]
                 data = data[:chunk_size]
@@ -228,160 +235,19 @@
             if self.content_length:
                 self.content_length -= chunk_size
                 if self.content_length <= 0:
-                    self.consumer.close()
-                    return True
-
-
-##
-# Asynchronous HTTP/1.1 client.
-
-class async_http(asyncore.dispatcher_with_send):
-    """Asynchronous HTTP client.
-    This client keeps a queue of files to download, and
-    tries to asynchronously (and pipelined) download them,
-    alerting the consumer as bits are downloaded.
-    """
-
-    max_requests = 4
-
-    def __init__(self, scheme, host):
-        """Connect to the given host, extra requests are made on the
-        add_request member function.
-
-        :param scheme: The connection method, such as http/https, currently
-                       we only support http
-        :param host:   The host to connect to, either a proxy, or the actual host.
-        """
-        asyncore.dispatcher_with_send.__init__(self)
-
-        # use origin host
-        self.scheme = scheme
-        self.host = host
-
-        assert scheme == "http", "only supports HTTP requests (%s)" % scheme
-
-        self._connected = False
-        self._queue = []
-        self._current = None
-
-    def _connect(self):
-        # get port number
-        host = self.host
-        try:
-            host, port = self.host.split(":", 1)
-            port = int(port)
-        except (TypeError, ValueError):
-            port = 80 # default port
-
-        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
-        try:
-            self.connect((host, port))
-            self._connected = True
-        except socket.error:
-            self.handle_error()
-
-    def _close(self):
-        if self._connected:
-            self.close()
-        self._connected = False
-
-    def _request_next(self):
-        extra_data = None
-        if self._current:
-            extra_data = self._current.data
-        if len(self._queue) > 0:
-            self._current = self._queue.pop(0)
-            if not self._current.requested:
-                self._current.http_request(self)
-            if extra_data:
-                self._update_current(extra_data)
-        else:
-            # TODO: Consider some sort of delayed closing,
-            # rather than closing the instant the
-            # queue is empty. But I don't know any way
-            # under async_core to be alerted at a later time.
-            # If this were a thread, I would sleep, waking
-            # up to check for more work, and eventually timing
-            # out and disconnecting.
-            self._close()
-
-    def _update_current(self, data):
-        res = self._current.add_data(data)
-        if res is None:
-            # Downloading is continuing
-            if self._current.original_content_length and self._current.http_1_1:
-                # We can pipeline our requests, since we
-                # are getting a content_length count back
-                for i, r in enumerate(self._queue[:self.max_requests-1]):
-                    if not r.requested:
-                        r.http_request(self)
-            return
-        if res:
-            # We finished downloading the last file
-            self._request_next()
-        else:
-            # There was a failure
-            self.handle_error()
-
-    def add_request(self, request):
-        """Add a new Request into the queue."""
-        self._queue.append(request)
-        if not self._connected:
-            self._connect()
-
-    def handle_connect(self):
-        self._request_next()
-
-    def handle_expt(self):
-        # connection failed (windows); notify consumer
-        assert self._current
-
-        if sys.platform == "win32":
-            self._close()
-            self._current.consumer.http(0, self._current)
-
-    def handle_read(self):
-        # handle incoming data
-        assert self._current
-
-        data = self.recv(2048)
-
-        self._update_current(data)
-
+                    return self.handle_close()
 
     def handle_close(self):
-        """When does this event occur? Is it okay to start the next entry in the queue
-        (possibly reconnecting), or is this an indication that we should stop?
-        """
-        if self._current:
-            self._current.consumer.close()
-        self._close()
-        if len(self._queue) > 0:
-            self._connect()
+        self.consumer.close()
+        self.close()
 
     def handle_error(self):
-        if self._current:
-            self._current.consumer.http(0, self._current, sys.exc_info())
-        # Should this be broadcast to all other items waiting in the queue?
-        self._close()
-
-_connections = {}
+        self.consumer.http(0, self, sys.exc_info())
+        self.close()
 
 def do_request(uri, consumer, extra_headers=None):
-    global _connections
-    request = Request(uri, consumer, extra_headers)
-
-    scheme = request.scheme
-    host = request.host
-    if request.proxy:
-        host = request.proxy
-    key = (scheme, host)
-    if not _connections.has_key(key):
-        _connections[key] = async_http(scheme, host)
-
-    _connections[key].add_request(request)
-
-    return request
+
+    return async_http(uri, consumer, extra_headers)
 
 if __name__ == "__main__":
     class dummy_consumer:
@@ -390,13 +256,13 @@
             print "feed", repr(data[:20]), repr(data[-20:]), len(data)
         def close(self):
             print "close"
-        def http(self, ok, connection, *args, **kwargs):
-            print ok, connection, args, kwargs
+        def http(self, ok, connection, **args):
+            print ok, connection, args
             print "status", connection.status
             print "header", connection.header
-    if len(sys.argv) < 2:
-        do_request('http://www.cnn.com/', dummy_consumer())
-    else:
-        for url in sys.argv[1:]:
-            do_request(url, dummy_consumer())
+    try:
+        url = sys.argv[1]
+    except IndexError:
+        url = "http://www.cnn.com/"
+    do_request(url, dummy_consumer())
     asyncore.loop()

